/* tclsrvrr - tcl interface to SRV RRs */

#include <tcl.h>
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include "srv.h"


static int
srvrr_getsrv (ClientData  unused,
              Tcl_Interp *interp,
              int         objc,
              Tcl_Obj    *CONST objv[]) {
    char           *proto;
    struct srvhost *s1,
                   *s2;
    Tcl_Obj        *resultPtr;
    
    switch (objc) {
        case 3:
            proto = "tcp";
            break;

        case 4:
            proto = Tcl_GetString (objv[3]);
            break;

        default:
            Tcl_WrongNumArgs (interp, 1, objv,
                              "domainName serviceName ?protocolName?");
            return TCL_ERROR;
    }

    if (!(s1 = getsrv (Tcl_GetString (objv[1]), Tcl_GetString (objv[2]),
                       proto))) {
        Tcl_SetResult (interp, (char *) hstrerror (h_errno), TCL_STATIC);
                          
        return TCL_ERROR;
    }

    resultPtr = Tcl_NewObj ();
    for (s2 = s1; s2; s2 = s2 -> next) {
        Tcl_Obj *elemPtr = Tcl_NewObj ();

        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewStringObj ("pref", -1));
        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewIntObj (s2 -> pref));

        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewStringObj ("weight", -1));
        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewIntObj (s2 -> weight));

        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewStringObj ("rweight", -1));
        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewIntObj (s2 -> rweight));

        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewStringObj ("port", -1));
        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewIntObj (s2 -> port));

        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewStringObj ("name", -1));
        Tcl_ListObjAppendElement (interp, elemPtr,
                                  Tcl_NewStringObj (s2 -> name, -1));

        Tcl_ListObjAppendElement (interp, resultPtr, elemPtr);
    }

    freesrvhost (s1);

    Tcl_SetObjResult (interp, resultPtr);
    return TCL_OK;
}


int
Tclsrvrr_Init (Tcl_Interp *interp) {
    if (Tcl_InitStubs (interp, "8.0", 0) == NULL)
        return TCL_ERROR;

    if ((Tcl_PkgRequire (interp, "Tcl", TCL_VERSION, 0) == NULL)
            && (TCL_VERSION[0] == '7')
            && (Tcl_PkgRequire (interp, "Tcl", "8.0", 0) == NULL))
        return TCL_ERROR;

    if (Tcl_PkgProvide (interp, "srvrr", VERSION) != TCL_OK)
        return TCL_ERROR;

    Tcl_CreateObjCommand (interp, "srvrr::getsrv", srvrr_getsrv,
                          (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

    srandom (getpid () * time (NULL));
    return TCL_OK;
}
